@@ -7,9 +7,8 @@ package Tickit::WidgetRole::Penable;
 
 use strict;
 use warnings;
-use base qw( Tickit::WidgetRole );
 
-our $VERSION = '0.29_002';
+our $VERSION = '0.18';
 
 use Carp;
 
@@ -51,9 +50,9 @@ have on initialisation.
 
 =cut
 
-sub export_subs_for
+sub import
 {
-   my $class = shift;
+   my $pkg = caller;
    shift;
    my %args = @_;
 
@@ -64,7 +63,7 @@ sub export_subs_for
 
    my $attr = "${name}_pen";
 
-   return {
+   my %subs = (
       "${name}_pen" => sub {
          my $self = shift;
          return $self->{$attr};
@@ -83,7 +82,10 @@ sub export_subs_for
          $self->{$attr} = $newpen;
          $newpen->add_on_changed( $self, $name );
       },
-   }
+   );
+
+   no strict 'refs';
+   *{"${pkg}::$_"} = $subs{$_} for keys %subs;
 }
 
 =head2 $pen = $widget->NAME_pen
