@@ -8,7 +8,7 @@ package Tickit::Window;
 use strict;
 use warnings;
 
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 
 use Carp;
 
@@ -145,6 +145,35 @@ sub make_float
    return $sub;
 }
 
+=head2 $popup = $win->make_popup( $top, $left, $lines, $cols )
+
+Constructs a new floating popup window starting at the given coordinates
+relative to this window. It will be sized to the given limits.
+
+This window will have the root window as its parent, rather than the window
+the method was called on. Additionally, a popup window will steal all keyboard
+and mouse events that happen, regardless of focus or mouse position. It is
+possible that, if the window has an C<on_mouse> handler, that it may receive
+mouse events from outwide the bounds of the window.
+
+=cut
+
+sub make_popup
+{
+   my $win = shift;
+   my ( $top, $left, $lines, $cols ) = @_;
+
+   while( $win->parent ) {
+      $top  += $win->top;
+      $left += $win->left;
+      $win = $win->parent;
+   }
+
+   my $popup = $win->make_float( $top, $left, $lines, $cols );
+   $popup->{steal_input} = 1;
+   return $popup;
+}
+
 sub _reap_dead_children
 {
    my $self = shift;
@@ -371,6 +400,12 @@ sub _handle_key
 
    return 0 unless $self->is_visible;
 
+   $self->_reap_dead_children;
+   my $children = $self->{child_windows};
+   if( $children and @$children and $children->[0]->{steal_input} ) {
+      $children->[0]->_handle_key( @_ ) and return;
+   }
+
    my $focus_child = $self->{focus_child};
    if( $focus_child ) {
       $focus_child->_handle_key( @_ ) and return 1;
@@ -380,7 +415,7 @@ sub _handle_key
       $on_key->( $self, @_ ) and return 1;
    }
 
-   if( my $children = $self->{child_windows} ) {
+   if( $children ) {
       foreach my $child ( @$children ) {
          next unless $child; # weakrefs; may be undef
          next if $focus_child and $child == $focus_child;
@@ -431,8 +466,10 @@ sub _handle_mouse
          my $child_line = $line - $child->top;
          my $child_col  = $col  - $child->left;
 
-         next if $child_line < 0 or $child_line >= $child->lines;
-         next if $child_col  < 0 or $child_col  >= $child->cols;
+         if( !$child->{steal_input} ) {
+            next if $child_line < 0 or $child_line >= $child->lines;
+            next if $child_col  < 0 or $child_col  >= $child->cols;
+         }
 
          return 1 if $child->_handle_mouse( $ev, $button, $child_line, $child_col );
       }
@@ -842,7 +879,7 @@ sub print
    my $self = shift;
    my $text = shift;
 
-   my $pen = ( @_ == 1 ) ? shift : Tickit::Pen->new( @_ );
+   my $pen = ( @_ == 1 ) ? shift->clone : Tickit::Pen->new( @_ );
 
    # First collect up the pen attributes and abort early if any window is
    # invisible
@@ -907,7 +944,7 @@ sub erasech
    my $count = shift;
    my $moveend = shift;
 
-   my $pen = ( @_ == 1 ) ? shift : Tickit::Pen->new( @_ );
+   my $pen = ( @_ == 1 ) ? shift->clone : Tickit::Pen->new( @_ );
 
    # First collect up the pen attributes and abort early if any window is
    # invisible
@@ -916,8 +953,6 @@ sub erasech
       $pen->default_from( $win->pen );
    }
 
-   my $bg = $pen->getattr( 'bg' );
-
    my $line = $self->{output_line};
    my $term = $self->term;
 
@@ -938,7 +973,7 @@ sub erasech
             $term->goto( $abs_top + $line, $abs_left + $self->{output_column} );
          }
 
-         $term->setpen( bg => $bg );
+         $term->setpen( $pen );
          $term->erasech( $len, $moveend );
       }
       else {
@@ -950,6 +985,37 @@ sub erasech
    }
 }
 
+=head2 $win->clearrect( $rect )
+
+=head2 $win->clearrect( $rect, $pen )
+
+=head2 $win->clearrect( $rect, %attrs )
+
+Erase the content of the window within the given L<Tickit::Rect>. If a
+C<Tickit::Pen> or pen attributes are provided, they are used to override the
+background colour for the erased region.
+
+=cut
+
+sub clearrect
+{
+   my $self = shift;
+   my $rect = shift;
+
+   my $pen = ( @_ == 1 ) ? shift->clone : Tickit::Pen->new( @_ );
+
+   if( $rect->top == 0 and $rect->left == 0 and
+       $rect->bottom == $self->lines and $rect->right == $self->cols ) {
+      $self->clear;
+      return;
+   }
+
+   foreach my $line ( $rect->linerange ) {
+      $self->goto( $line, $rect->left );
+      $self->erasech( $rect->cols, undef, $pen );
+   }
+}
+
 =head2 $success = $win->scrollrect( $top, $left, $lines, $cols, $downward, $rightward )
 
 =head2 $success = $win->scrollrect( ..., $pen )
@@ -975,35 +1041,42 @@ scroll the terminal (and returns false).
 
 sub scrollrect
 {
-   my $self = shift;
+   my $win = shift;
    my ( $top, $left, $lines, $cols, $downward, $rightward, @args ) = @_;
 
-   $top  >= 0 and $top  < $self->lines or croak '$top out of bounds';
-   $left >= 0 and $left < $self->cols  or croak '$left out of bounds';
+   my $pen = ( @args == 1 ) ? $args[0]->clone : Tickit::Pen->new( @args );
 
-   $lines > 0 and $top + $lines <= $self->lines or croak '$lines out of bounds';
-   $cols  > 0 and $left + $cols <= $self->cols  or croak '$cols out of bounds';
+   while( $win ) {
+      $top  >= 0 and $top  < $win->lines or croak '$top out of bounds';
+      $left >= 0 and $left < $win->cols  or croak '$left out of bounds';
 
-   return unless $self->{visible};
+      $lines > 0 and $top + $lines <= $win->lines or croak '$lines out of bounds';
+      $cols  > 0 and $left + $cols <= $win->cols  or croak '$cols out of bounds';
 
-   my $pen = ( @args == 1 ) ? $args[0] : Tickit::Pen->new( @args );
+      return unless $win->{visible};
 
-   $pen->default_from( $self->pen );
+      $pen->default_from( $win->pen );
 
-   if( my $parent = $self->parent ) {
-      return $parent->scrollrect(
-         $self->top  + $top, $self->left + $left, $lines, $cols,
-         $downward, $rightward,
-         $pen,
-      );
+      $top  += $win->top;
+      $left += $win->left;
+
+      my $parent = $win->parent or last;
+      $win = $parent;
    }
-   else {
-      $self->term->setpen( bg => $pen->getattr( 'bg' ) );
-      return $self->term->scrollrect(
-         $top, $left, $lines, $cols,
-         $downward, $rightward
-      );
+
+   # Check there are no floating windows in the way
+   foreach my $line ( $top .. $top + $lines - 1 ) {
+      my ( $vis, $len ) = $win->_get_span_visibility( $line, $left );
+      return 0 unless $vis;
+      return 0 unless $len >= $cols;
    }
+
+   my $term = $win->term;
+
+   $term->setpen( bg => $pen->getattr( 'bg' ) );
+
+   return $term->scrollrect( $top, $left, $lines, $cols,
+      $downward, $rightward );
 }
 
 =head2 $success = $win->scroll( $downward, $rightward )
@@ -1092,19 +1165,23 @@ sub restore
    my $self = shift;
    my $root = $self->root;
 
+   my $term = $root->term;
+
    if( my $focus_child = $root->{focus_child} ) {
       if( $focus_child->is_visible ) {
-         $root->term->mode_cursorvis( 1 );
+         $term->mode_cursorvis( 1 );
          $focus_child->_gain_focus;
       }
       else {
-         $root->term->mode_cursorvis( 0 );
+         $term->mode_cursorvis( 0 );
       }
    }
    elsif( defined $root->{focus_line} ) {
-      $root->term->mode_cursorvis( 1 );
+      $term->mode_cursorvis( 1 );
       $root->_gain_focus;
    }
+
+   $term->flush;
 }
 
 =head2 $win->clearline( $line )
